local assets=
{ 
    Asset("ANIM", "anim/creditcard.zip"),
    Asset("ANIM", "anim/swap_creditcard.zip"), 

    Asset("ATLAS", "images/inventoryimages/creditcard.xml"),
    Asset("IMAGE", "images/inventoryimages/creditcard.tex"),
}

local prefabs = 
{
}

--TUNING.CREDITCARD_UPGRADABLE -- defined in modmain
TUNING.CREDITCARD_DAMAGE = 0
TUNING.CREDITCARD_RANGE = 0
TUNING.CREDITCARD_LIMIT = 90
TUNING.CREDITCARD_PAYPERDAY = 2
TUNING.CREDITCARD_PENALTY = -10

local UPGRADE = {
    {
        times = 50,
        limit = 90+30,
        payperday = 2,
    },
    {
        times = 125,
        limit = 90+60,
        payperday = 2,
    },
    {
        times = 320,
        limit = 90+90,
        payperday = 2,
    },
    {
        times = 800,
        limit = 90+120,
        payperday = 3,
    },
}

local function setStatus(inst)
    -- upgrade it
    if TUNING.CREDITCARD_UPGRADABLE then
        print("---- creditcard setStatus(), upgrade")
        for _,data in pairs(UPGRADE) do
            if inst.usetimes >= data.times then
                TUNING.CREDITCARD_LIMIT = data.limit
                TUNING.CREDITCARD_PAYPERDAY = data.payperday
            end
        end
    end

    local s= "Debt:"..inst.debtAmount.."/"..TUNING.CREDITCARD_LIMIT..", Repay:"..TUNING.CREDITCARD_PAYPERDAY.."oinc/day"
    inst.components.inspectable:SetDescription(s)
    inst.components.weapon:SetDamage(inst.debtAmount)
end


local function getMoney(inst)
    local money = math.max(TUNING.CREDITCARD_LIMIT - inst.debtAmount, 0)
--    print("Credit Card getMoney(), money is "..money)
    return money
end


local function payMoney(inst, cost)
    local left = math.max(cost - getMoney(inst), 0)
    inst.debtAmount = math.min(inst.debtAmount + cost, TUNING.CREDITCARD_LIMIT)

    -- increment usetimes
    if left < cost then inst.usetimes = inst.usetimes + 1 end

    setStatus(inst)

--    print("Credit Card payMoney(), left is "..left)
    return left
end


local function setupShopper(inst, owner)
    local shopper = owner.components.shopper
    if shopper then
        -- overwrite shopper:PayMoney()
        if not shopper.creditcard_fn01 then 
--            print("---- Credit Card setupShopper(), overwrite shopper:PayMoney()")
            shopper.creditcard_fn01 = shopper.PayMoney
            function shopper:PayMoney(inventory,cost)
                local left = cost
                if inst.components.equippable:IsEquipped() then
                    left = payMoney(inst, left)
                end
                shopper:creditcard_fn01(inventory,left)
            end
        end

        -- overwrite shopper:GetMoney()
        if not shopper.creditcard_fn02 then
--            print("---- Credit Card setupShopper(), overwrite shopper:GetMoney()")
            shopper.creditcard_fn02 = shopper.GetMoney
            function shopper:GetMoney(inventory)
                local money = 0
                if inst.components.equippable:IsEquipped() then
                    money = getMoney(inst)
                end
                money = money + shopper:creditcard_fn02(inventory)
                return money
            end
        end
    end
end


local function onEquip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_object", "swap_creditcard", "wand")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 

    setupShopper(inst, owner)
end


local function onUnequip(inst, owner) 
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end


local function onSave(inst, data)
    data.usetimes = inst.usetimes
    data.debtAmount = inst.debtAmount
end


local function onLoad(inst, data)
    if data then
        inst.usetimes = data.usetimes or 0
        inst.debtAmount = data.debtAmount or 0
    end
    setStatus(inst)
end


local function doRepayment(inst)
    if inst.debtAmount > 0 then
--        print("---- Credit Card doRepayment()")
        local paid = false
        local owner = GetPlayer()
        if owner then
            setupShopper(inst, owner) -- if not setUpped
            local inventory = owner.components.inventory
            for i=1, TUNING.CREDITCARD_PAYPERDAY do
                if owner.components.shopper:creditcard_fn02(inventory) >= 1 then
                    owner.components.shopper:creditcard_fn01(inventory,1)
                    inst.debtAmount = inst.debtAmount - 1 
                    if owner.components.talker then
                        owner.components.talker:Say("Yes, I should repay my debt.")
                    end
                    paid = true
--                    print("\tpaid 1oinc")
                else
                    if owner.components.talker then
                        owner.components.talker:Say("I couldn't repay my dept for today...")
                    end
                    owner.components.sanity:DoDelta(TUNING.CREDITCARD_PENALTY)
--                    print("\tcould not pay!")
                end
            end
        end

        if paid then
            inst.SoundEmitter:PlaySound("dontstarve_DLC003/common/objects/coins/1")
            setStatus(inst)
        end
    end
end


local function repayOinc(inst,giver,item)
    if item and item.prefab == "oinc" then
        inst.debtAmount = inst.debtAmount - 1
        inst.SoundEmitter:PlaySound("dontstarve_DLC003/common/objects/coins/1")
        setStatus(inst)
    end
end


local function fn(colour)

    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    
    anim:SetBank("creditcard")
    anim:SetBuild("creditcard")
    anim:PlayAnimation("idle")

    inst:AddTag("sharp")

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.CREDITCARD_DAMAGE)
	inst.components.weapon:SetRange(TUNING.CREDITCARD_RANGE)

	inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.imagename = "creditcard"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/creditcard.xml"
    
    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onEquip)
    inst.components.equippable:SetOnUnequip(onUnequip)
--[[
    function inst.components.equippable:IsEquipped()
        local player = GetPlayer()
        if player then
            return player.components.inventory:IsItemEquipped(inst)
        else
            return false
        end
    end
]]

    inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(function(inst, item) return inst.debtAmount > 0 and item and (item.prefab=="oinc") end)
    inst.components.trader.onaccept = repayOinc
    inst.components.trader.onrefuse = function(inst,giver,item) if giver.components.talker then giver.components.talker:Say("I should repay my debt by design.") end end


    inst:ListenForEvent("daycomplete", function() doRepayment(inst) end, GetWorld())

    inst.usetimes = 0
    inst.debtAmount = 0
    inst.OnSave = onSave
    inst.OnLoad = onLoad

    return inst
end

-- Add some strings for this item
STRINGS.NAMES.CREDITCARD = "Credit Card"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.CREDITCARD = "Credit limit is 90 oinc, Repayment is 2 oinc/day."

return  Prefab("common/inventory/creditcard", fn, assets, prefabs)
